/*

various functions

*/
// includes

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/sleep.h>
#include <util/atomic.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "main.h"
#include <util/delay.h>
#include "globals.h"
#include "uart.h"
#include "functions.h"

int putchar__(char c, FILE *stream) {
	uart_putc(c);
	return 0;
}

void cfg_read(void) {
    eeprom_read_block(&gconfig, &gconfig_ee, sizeof(gconfig));
    gtime = gconfig;
}

void cfg_write(void) {
    eeprom_update_block(&gconfig, &gconfig_ee, sizeof(gconfig));
}

void set_DDS(uint16_t framerate) {
    uint32_t f_max;

    // max. DDS frequency without additional software prescaler
    f_max = (gconfig.f_cpu / 65536) + 1;     
    
    gDDSprescaler = (f_max / framerate) + 1;
    gDDSfrequency = framerate * gDDSprescaler;
    gDDSincrement = (gconfig.f_cpu / gDDSfrequency) - 1;    // period is OCR1A +1
    gDDSremainder = gconfig.f_cpu % gDDSfrequency;
}

void set_SYNC(uint8_t mode) {

    switch (mode) {
        case MODE_EXT_RISE:
            SYNC_EXT_RISE
            TIMSK &= ~(1<<OCIE1A);      // disable timer interrupt
            GIFR   =  (1<<INTF1);       // clear pending INT1 interrupt
            GICR  |=  (1<<INT1);        // enable INT1
        break;

        case MODE_EXT_FALL:
            SYNC_EXT_FALL
            TIMSK &= ~(1<<OCIE1A);      // disable timer interrupt
            GIFR   =  (1<<INTF1);       // clear pending INT1 interrupt
            GICR  |=  (1<<INT1);        // enable INT1
        break;

        default:
            SYNC_INTERNAL
            GICR  &= ~(1<<INT1);        // disable INT1
            TIFR   =  (1<<OCF1A);       // clear pending timer interrupt
            TIMSK |=  (1<<OCIE1A);      // enable timer interrupt
        break;
    }
}

//convert global time to led code
/*
void time2led(void) {

    uint16_t tmp;
 
    tmp = gtime.fff;  
    gTLC59284_data[0] = hex2led[tmp % 10]; tmp /=10; 
    gTLC59284_data[1] = hex2led[tmp % 10]; tmp /=10; 
    gTLC59284_data[2] = hex2led[tmp];
     
    tmp = gtime.ss; 
    gTLC59284_data[3] = hex2led[tmp % 10] | DOT; tmp /=10; 
    gTLC59284_data[4] = hex2led[tmp];
     
    tmp = gtime.mm; 
    gTLC59284_data[5] = hex2led[tmp % 10] | DOT; tmp /=10; 
    gTLC59284_data[6] = hex2led[tmp];
     
    tmp = gtime.hh; 
    gTLC59284_data[7] = hex2led[tmp % 10] | DOT; tmp /=10; 
    gTLC59284_data[8] = hex2led[tmp]; 
}
*/

// fast version, uses quotient and remainder of division

void time2led(void) {

    div_t tmp;
 
    tmp = div(gtime.fff, 10);  
    gTLC59284_data[0] = hex2led[tmp.rem];
    tmp = div(tmp.quot, 10); 
    gTLC59284_data[1] = hex2led[tmp.rem];
    gTLC59284_data[2] = hex2led[tmp.quot];
 
    tmp = div(gtime.ss, 10);
    gTLC59284_data[3] = hex2led[tmp.rem] | DOT;
    gTLC59284_data[4] = hex2led[tmp.quot];
     
    tmp = div(gtime.mm, 10); 
    gTLC59284_data[5] = hex2led[tmp.rem] | DOT;
    gTLC59284_data[6] = hex2led[tmp.quot];
     
    tmp = div(gtime.hh, 10); 
    gTLC59284_data[7] = hex2led[tmp.rem] | DOT;
    gTLC59284_data[8] = hex2led[tmp.quot]; 
}

void print_cfg(void) {
    int32_t ppm;
    char sign='+';

    PRINT("Framerate  : %4d Hz\r\nSync       : ", gconfig.framerate);

    switch(gconfig.mode) {
        case MODE_EXT_RISE: PRINT("external, rising edge"); break;
        case MODE_EXT_FALL: PRINT("external, falling edge"); break;
        default:            PRINT("internal"); break;
    }

    ppm = ((gconfig.f_cpu - F_CPU)*10) / (F_CPU / 1000000);
    if (ppm <0) sign = '-';
    PRINT("\r\nCrystal    : %c%d.%d ppm\r\n", sign, abs(ppm/10), abs(ppm%10));
    PRINT("Start Time : %02d:%02d:%02d:%03d\r\n", gconfig.hh, gconfig.mm, gconfig.ss, gconfig.fff);
}

void increment_timer(void) {
    gtime.fff++;
    if (gtime.fff >= gconfig.framerate) {
        gtime.fff =0; 
        gtime.ss++; 
        if (gtime.ss >= 60) { 
            gtime.ss =0; 
            gtime.mm++; 
            if (gtime.mm >= 60) { 
                gtime.mm =0; 
                gtime.hh++; 
                if (gtime.hh >= 24) { 
                    gtime.hh =0; 
                } 
            } 
        } 
    }
}

// update TLC59284 data

void TLC59284_update(uint8_t *data) {
    uint8_t i, tmp; 

    SPDR = *data++;
    for (i=1; i<9; i++) {
        tmp = *data++;
        while(!(SPSR & (1<<SPIF)));
        SPDR = tmp;
    }
    while(!(SPSR & (1<<SPIF)));
}

// simple animation player

void animation(uint16_t *ani_data, uint16_t cnt) {
    uint16_t ani;
    uint8_t i, j, pos, code;

    for (i=0; i<cnt; i++) {
        for(j=0; j<9; j++) {
            gTLC59284_data[j]=0;
        }
        ani = pgm_read_word(&ani_data[i]);
        pos = ani >> 8;
        code = ani & 0xFF;
        gTLC59284_data[pos] = code;
        TLC59284_update(gTLC59284_data);
        LAT_PULSE
        _delay_ms(100);
        
    }
}
